﻿using System;
using System.Diagnostics;
using System.Threading;

namespace TP12
{
	public class ProgramInput : Input
	{

		public Action NextAction { get; set; }
		Process process;
		Map map;
		bool error = false;
		int timeout;

		~ProgramInput()
		{
			try
			{
				process.Kill();
			}
			catch
			{
			}
		}

		public ProgramInput(string name, Map map, int x, int y, int timeout)
		{
			ProcessStartInfo info = new ProcessStartInfo(name);

			this.timeout = timeout;
			this.map = map;

			info.UseShellExecute = false;
			info.RedirectStandardOutput = true;
			info.RedirectStandardInput = true;
			try
			{
				process = Process.Start(info);
				process.StandardInput.WriteLine(timeout);
				process.StandardInput.WriteLine(map.Width
												+ " " + map.Height);
				process.StandardInput.WriteLine(x + " " + y);
				process.StandardInput.WriteLine(map.players.Count);
				foreach (var it in map.players)
				{
					process.StandardInput.WriteLine(it.x + " " + it.y);
				}
			}
			catch (Exception e)
			{
				Console.Error.WriteLine(e.Message);
			}
		}

		void RunThread()
		{
			try
			{
				process.StandardInput.WriteLine(map.players.Count);
				foreach (var it in map.players)
				{
					process.StandardInput.WriteLine(it.x + " " + it.y);
				}

				string line = process.StandardOutput.ReadLine();
				error = false;
				switch (line)
				{
					case "L":
						NextAction = Action.left;
						break;
					case "R":
						NextAction = Action.right;
						break;
					case "U":
						NextAction = Action.up;
						break;
					case "D":
						NextAction = Action.down;
						break;
					default:
						error = true;
						Console.Error.WriteLine("Invalid command: " + line);
						break;
				}
			}
			catch (Exception e)
			{
				Console.Error.WriteLine(e.Message);
			}
		}

		public override Action GetNextAction()
		{
			try
			{
				if (process.HasExited)
					return NextAction;
				error = true;
				Thread thread = new Thread(this.RunThread);
				thread.Start();

				Stopwatch timer = new Stopwatch();
				timer.Start();
				while (thread.IsAlive && timer.ElapsedMilliseconds < timeout)
				{
					// process.Thread is only partially implemented in Mono
					// but the thread count (what we use here) works fine
					if (process.Threads.Count > 16)
					{
						Console.Error.WriteLine("Too many threads!");
						process.Kill();
						break;
					}
				}


				thread.Join(0);
				if (error)
				{
					try
					{
						process.Kill();
					}
					catch
					{
					}
				}
				return NextAction;
			}
			catch (Exception e)
			{
				Console.Error.WriteLine(e);
				return NextAction;
			}
		}
	}
}
